# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.name import Name
from paypalserversdk.models.phone_with_type import PhoneWithType


class PaypalWalletCustomerRequest(object):

    """Implementation of the 'PayPal Wallet Customer Request' model.

    Attributes:
        id (str): The unique ID for a customer generated by PayPal.
        email_address (str): The internationalized email address. Note: Up to
            64 characters are allowed before and 255 characters are allowed
            after the @ sign. However, the generally accepted maximum length
            for an email address is 254 characters. The pattern verifies that
            an unquoted @ sign exists.
        phone (PhoneWithType): The phone information.
        name (Name): The name of the party.
        merchant_customer_id (str): Merchants and partners may already have a
            data-store where their customer information is persisted. Use
            merchant_customer_id to associate the PayPal-generated customer.id
            to your representation of a customer.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "email_address": 'email_address',
        "phone": 'phone',
        "name": 'name',
        "merchant_customer_id": 'merchant_customer_id'
    }

    _optionals = [
        'id',
        'email_address',
        'phone',
        'name',
        'merchant_customer_id',
    ]

    def __init__(self,
                 id=APIHelper.SKIP,
                 email_address=APIHelper.SKIP,
                 phone=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 merchant_customer_id=APIHelper.SKIP):
        """Constructor for the PaypalWalletCustomerRequest class"""

        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id 
        if email_address is not APIHelper.SKIP:
            self.email_address = email_address 
        if phone is not APIHelper.SKIP:
            self.phone = phone 
        if name is not APIHelper.SKIP:
            self.name = name 
        if merchant_customer_id is not APIHelper.SKIP:
            self.merchant_customer_id = merchant_customer_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        email_address = dictionary.get("email_address") if dictionary.get("email_address") else APIHelper.SKIP
        phone = PhoneWithType.from_dictionary(dictionary.get('phone')) if 'phone' in dictionary.keys() else APIHelper.SKIP
        name = Name.from_dictionary(dictionary.get('name')) if 'name' in dictionary.keys() else APIHelper.SKIP
        merchant_customer_id = dictionary.get("merchant_customer_id") if dictionary.get("merchant_customer_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   email_address,
                   phone,
                   name,
                   merchant_customer_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'email_address={(self.email_address if hasattr(self, "email_address") else None)!r}, '
                f'phone={(self.phone if hasattr(self, "phone") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'merchant_customer_id={(self.merchant_customer_id if hasattr(self, "merchant_customer_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'email_address={(self.email_address if hasattr(self, "email_address") else None)!s}, '
                f'phone={(self.phone if hasattr(self, "phone") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'merchant_customer_id={(self.merchant_customer_id if hasattr(self, "merchant_customer_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class CustomerResponse(object):

    """Implementation of the 'Customer Response' model.

    Customer in merchant's or partner's system of records.

    Attributes:
        id (str): The unique ID for a customer generated by PayPal.
        merchant_customer_id (str): Merchants and partners may already have a
            data-store where their customer information is persisted. Use
            merchant_customer_id to associate the PayPal-generated customer.id
            to your representation of a customer.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "merchant_customer_id": 'merchant_customer_id'
    }

    _optionals = [
        'id',
        'merchant_customer_id',
    ]

    def __init__(self,
                 id=APIHelper.SKIP,
                 merchant_customer_id=APIHelper.SKIP):
        """Constructor for the CustomerResponse class"""

        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id 
        if merchant_customer_id is not APIHelper.SKIP:
            self.merchant_customer_id = merchant_customer_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        merchant_customer_id = dictionary.get("merchant_customer_id") if dictionary.get("merchant_customer_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   merchant_customer_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'merchant_customer_id={(self.merchant_customer_id if hasattr(self, "merchant_customer_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'merchant_customer_id={(self.merchant_customer_id if hasattr(self, "merchant_customer_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class FailedPaymentDetails(object):

    """Implementation of the 'Failed Payment Details' model.

    The details for the failed payment of the subscription.

    Attributes:
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.
        time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        reason_code (ReasonCode): The reason code for the payment failure.
        next_payment_retry_time (str): The date and time, in [Internet date
            and time format](https://tools.ietf.org/html/rfc3339#section-5.6).
            Seconds are required while fractional seconds are optional. Note:
            The regular expression provides guidance but does not reject all
            invalid dates.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount": 'amount',
        "time": 'time',
        "reason_code": 'reason_code',
        "next_payment_retry_time": 'next_payment_retry_time'
    }

    _optionals = [
        'reason_code',
        'next_payment_retry_time',
    ]

    def __init__(self,
                 amount=None,
                 time=None,
                 reason_code=APIHelper.SKIP,
                 next_payment_retry_time=APIHelper.SKIP):
        """Constructor for the FailedPaymentDetails class"""

        # Initialize members of the class
        self.amount = amount 
        self.time = time 
        if reason_code is not APIHelper.SKIP:
            self.reason_code = reason_code 
        if next_payment_retry_time is not APIHelper.SKIP:
            self.next_payment_retry_time = next_payment_retry_time 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = Money.from_dictionary(dictionary.get('amount')) if dictionary.get('amount') else None
        time = dictionary.get("time") if dictionary.get("time") else None
        reason_code = dictionary.get("reason_code") if dictionary.get("reason_code") else APIHelper.SKIP
        next_payment_retry_time = dictionary.get("next_payment_retry_time") if dictionary.get("next_payment_retry_time") else APIHelper.SKIP
        # Return an object of this model
        return cls(amount,
                   time,
                   reason_code,
                   next_payment_retry_time)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'amount={self.amount!r}, '
                f'time={self.time!r}, '
                f'reason_code={(self.reason_code if hasattr(self, "reason_code") else None)!r}, '
                f'next_payment_retry_time={(self.next_payment_retry_time if hasattr(self, "next_payment_retry_time") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'amount={self.amount!s}, '
                f'time={self.time!s}, '
                f'reason_code={(self.reason_code if hasattr(self, "reason_code") else None)!s}, '
                f'next_payment_retry_time={(self.next_payment_retry_time if hasattr(self, "next_payment_retry_time") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class PricingScheme(object):

    """Implementation of the 'Pricing Scheme' model.

    The pricing scheme details.

    Attributes:
        price (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.
        pricing_model (PricingModel): The pricing model for the billing cycle.
        reload_threshold_amount (Money): The currency and amount for a
            financial transaction, such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "pricing_model": 'pricing_model',
        "price": 'price',
        "reload_threshold_amount": 'reload_threshold_amount'
    }

    _optionals = [
        'price',
        'reload_threshold_amount',
    ]

    def __init__(self,
                 pricing_model=None,
                 price=APIHelper.SKIP,
                 reload_threshold_amount=APIHelper.SKIP):
        """Constructor for the PricingScheme class"""

        # Initialize members of the class
        if price is not APIHelper.SKIP:
            self.price = price 
        self.pricing_model = pricing_model 
        if reload_threshold_amount is not APIHelper.SKIP:
            self.reload_threshold_amount = reload_threshold_amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        pricing_model = dictionary.get("pricing_model") if dictionary.get("pricing_model") else None
        price = Money.from_dictionary(dictionary.get('price')) if 'price' in dictionary.keys() else APIHelper.SKIP
        reload_threshold_amount = Money.from_dictionary(dictionary.get('reload_threshold_amount')) if 'reload_threshold_amount' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(pricing_model,
                   price,
                   reload_threshold_amount)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'price={(self.price if hasattr(self, "price") else None)!r}, '
                f'pricing_model={self.pricing_model!r}, '
                f'reload_threshold_amount={(self.reload_threshold_amount if hasattr(self, "reload_threshold_amount") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'price={(self.price if hasattr(self, "price") else None)!s}, '
                f'pricing_model={self.pricing_model!s}, '
                f'reload_threshold_amount={(self.reload_threshold_amount if hasattr(self, "reload_threshold_amount") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class OneTimeCharge(object):

    """Implementation of the 'One Time Charge' model.

    The one-time charge info at the time of checkout.

    Attributes:
        setup_fee (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        taxes (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.
        product_price (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        subtotal (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.
        total_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "total_amount": 'total_amount',
        "setup_fee": 'setup_fee',
        "shipping_amount": 'shipping_amount',
        "taxes": 'taxes',
        "product_price": 'product_price',
        "subtotal": 'subtotal'
    }

    _optionals = [
        'setup_fee',
        'shipping_amount',
        'taxes',
        'product_price',
        'subtotal',
    ]

    def __init__(self,
                 total_amount=None,
                 setup_fee=APIHelper.SKIP,
                 shipping_amount=APIHelper.SKIP,
                 taxes=APIHelper.SKIP,
                 product_price=APIHelper.SKIP,
                 subtotal=APIHelper.SKIP):
        """Constructor for the OneTimeCharge class"""

        # Initialize members of the class
        if setup_fee is not APIHelper.SKIP:
            self.setup_fee = setup_fee 
        if shipping_amount is not APIHelper.SKIP:
            self.shipping_amount = shipping_amount 
        if taxes is not APIHelper.SKIP:
            self.taxes = taxes 
        if product_price is not APIHelper.SKIP:
            self.product_price = product_price 
        if subtotal is not APIHelper.SKIP:
            self.subtotal = subtotal 
        self.total_amount = total_amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        total_amount = Money.from_dictionary(dictionary.get('total_amount')) if dictionary.get('total_amount') else None
        setup_fee = Money.from_dictionary(dictionary.get('setup_fee')) if 'setup_fee' in dictionary.keys() else APIHelper.SKIP
        shipping_amount = Money.from_dictionary(dictionary.get('shipping_amount')) if 'shipping_amount' in dictionary.keys() else APIHelper.SKIP
        taxes = Money.from_dictionary(dictionary.get('taxes')) if 'taxes' in dictionary.keys() else APIHelper.SKIP
        product_price = Money.from_dictionary(dictionary.get('product_price')) if 'product_price' in dictionary.keys() else APIHelper.SKIP
        subtotal = Money.from_dictionary(dictionary.get('subtotal')) if 'subtotal' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(total_amount,
                   setup_fee,
                   shipping_amount,
                   taxes,
                   product_price,
                   subtotal)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'setup_fee={(self.setup_fee if hasattr(self, "setup_fee") else None)!r}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!r}, '
                f'taxes={(self.taxes if hasattr(self, "taxes") else None)!r}, '
                f'product_price={(self.product_price if hasattr(self, "product_price") else None)!r}, '
                f'subtotal={(self.subtotal if hasattr(self, "subtotal") else None)!r}, '
                f'total_amount={self.total_amount!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'setup_fee={(self.setup_fee if hasattr(self, "setup_fee") else None)!s}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!s}, '
                f'taxes={(self.taxes if hasattr(self, "taxes") else None)!s}, '
                f'product_price={(self.product_price if hasattr(self, "product_price") else None)!s}, '
                f'subtotal={(self.subtotal if hasattr(self, "subtotal") else None)!s}, '
                f'total_amount={self.total_amount!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class PaymentMethod(object):

    """Implementation of the 'Payment Method' model.

    The customer and merchant payment preferences.

    Attributes:
        payee_preferred (PayeePaymentMethodPreference): The merchant-preferred
            payment methods.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "payee_preferred": 'payee_preferred'
    }

    _optionals = [
        'payee_preferred',
    ]

    def __init__(self,
                 payee_preferred='UNRESTRICTED'):
        """Constructor for the PaymentMethod class"""

        # Initialize members of the class
        self.payee_preferred = payee_preferred 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        payee_preferred = dictionary.get("payee_preferred") if dictionary.get("payee_preferred") else 'UNRESTRICTED'
        # Return an object of this model
        return cls(payee_preferred)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'payee_preferred={(self.payee_preferred if hasattr(self, "payee_preferred") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'payee_preferred={(self.payee_preferred if hasattr(self, "payee_preferred") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.subscription_amount_with_breakdown import SubscriptionAmountWithBreakdown
from paypalserversdk.models.subscription_payer_name import SubscriptionPayerName


class SubscriptionTransactionDetails(object):

    """Implementation of the 'Subscription Transaction Details' model.

    The transaction details.

    Attributes:
        status (CaptureStatus): The status of the captured payment.
        id (str): The PayPal-generated transaction ID.
        amount_with_breakdown (SubscriptionAmountWithBreakdown): The breakdown
            details for the amount. Includes the gross, tax, fee, and shipping
            amounts.
        payer_name (SubscriptionPayerName): The name of the party.
        payer_email (str): The internationalized email address. Note: Up to 64
            characters are allowed before and 255 characters are allowed after
            the @ sign. However, the generally accepted maximum length for an
            email address is 254 characters. The pattern verifies that an
            unquoted @ sign exists.
        time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "amount_with_breakdown": 'amount_with_breakdown',
        "time": 'time',
        "status": 'status',
        "payer_name": 'payer_name',
        "payer_email": 'payer_email'
    }

    _optionals = [
        'status',
        'payer_name',
        'payer_email',
    ]

    def __init__(self,
                 id=None,
                 amount_with_breakdown=None,
                 time=None,
                 status=APIHelper.SKIP,
                 payer_name=APIHelper.SKIP,
                 payer_email=APIHelper.SKIP):
        """Constructor for the SubscriptionTransactionDetails class"""

        # Initialize members of the class
        if status is not APIHelper.SKIP:
            self.status = status 
        self.id = id 
        self.amount_with_breakdown = amount_with_breakdown 
        if payer_name is not APIHelper.SKIP:
            self.payer_name = payer_name 
        if payer_email is not APIHelper.SKIP:
            self.payer_email = payer_email 
        self.time = time 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if dictionary.get("id") else None
        amount_with_breakdown = SubscriptionAmountWithBreakdown.from_dictionary(dictionary.get('amount_with_breakdown')) if dictionary.get('amount_with_breakdown') else None
        time = dictionary.get("time") if dictionary.get("time") else None
        status = dictionary.get("status") if dictionary.get("status") else APIHelper.SKIP
        payer_name = SubscriptionPayerName.from_dictionary(dictionary.get('payer_name')) if 'payer_name' in dictionary.keys() else APIHelper.SKIP
        payer_email = dictionary.get("payer_email") if dictionary.get("payer_email") else APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   amount_with_breakdown,
                   time,
                   status,
                   payer_name,
                   payer_email)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'status={(self.status if hasattr(self, "status") else None)!r}, '
                f'id={self.id!r}, '
                f'amount_with_breakdown={self.amount_with_breakdown!r}, '
                f'payer_name={(self.payer_name if hasattr(self, "payer_name") else None)!r}, '
                f'payer_email={(self.payer_email if hasattr(self, "payer_email") else None)!r}, '
                f'time={self.time!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'status={(self.status if hasattr(self, "status") else None)!s}, '
                f'id={self.id!s}, '
                f'amount_with_breakdown={self.amount_with_breakdown!s}, '
                f'payer_name={(self.payer_name if hasattr(self, "payer_name") else None)!s}, '
                f'payer_email={(self.payer_email if hasattr(self, "payer_email") else None)!s}, '
                f'time={self.time!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class StoreInformation(object):

    """Implementation of the 'Store Information' model.

    The store information.

    Attributes:
        store_id (str): The ID of a store for a merchant in the system of
            record.
        terminal_id (str): The terminal ID for the checkout stand in a
            merchant store.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "store_id": 'store_id',
        "terminal_id": 'terminal_id'
    }

    _optionals = [
        'store_id',
        'terminal_id',
    ]

    def __init__(self,
                 store_id=APIHelper.SKIP,
                 terminal_id=APIHelper.SKIP):
        """Constructor for the StoreInformation class"""

        # Initialize members of the class
        if store_id is not APIHelper.SKIP:
            self.store_id = store_id 
        if terminal_id is not APIHelper.SKIP:
            self.terminal_id = terminal_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        store_id = dictionary.get("store_id") if dictionary.get("store_id") else APIHelper.SKIP
        terminal_id = dictionary.get("terminal_id") if dictionary.get("terminal_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(store_id,
                   terminal_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'store_id={(self.store_id if hasattr(self, "store_id") else None)!r}, '
                f'terminal_id={(self.terminal_id if hasattr(self, "terminal_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'store_id={(self.store_id if hasattr(self, "store_id") else None)!s}, '
                f'terminal_id={(self.terminal_id if hasattr(self, "terminal_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.payment_method import PaymentMethod


class SubscriptionPatchApplicationContext(object):

    """Implementation of the 'Subscription Patch Application Context' model.

    The application context, which customizes the payer experience during the
    subscription approval process with PayPal.

    Attributes:
        brand_name (str): The label that overrides the business name in the
            PayPal account on the PayPal site.
        locale (str): The BCP 47-formatted locale of pages that the PayPal
            payment experience shows. PayPal supports a five-character code.
            For example, `da-DK`, `he-IL`, `id-ID`, `ja-JP`, `no-NO`, `pt-BR`,
            `ru-RU`, `sv-SE`, `th-TH`, `zh-CN`, `zh-HK`, or `zh-TW`.
        shipping_preference (ExperienceContextShippingPreference): The
            location from which the shipping address is derived.
        payment_method (PaymentMethod): The customer and merchant payment
            preferences.
        return_url (str): The URL where the customer is redirected after the
            customer approves the payment.
        cancel_url (str): The URL where the customer is redirected after the
            customer cancels the payment.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "return_url": 'return_url',
        "cancel_url": 'cancel_url',
        "brand_name": 'brand_name',
        "locale": 'locale',
        "shipping_preference": 'shipping_preference',
        "payment_method": 'payment_method'
    }

    _optionals = [
        'brand_name',
        'locale',
        'shipping_preference',
        'payment_method',
    ]

    def __init__(self,
                 return_url=None,
                 cancel_url=None,
                 brand_name=APIHelper.SKIP,
                 locale=APIHelper.SKIP,
                 shipping_preference='GET_FROM_FILE',
                 payment_method=APIHelper.SKIP):
        """Constructor for the SubscriptionPatchApplicationContext class"""

        # Initialize members of the class
        if brand_name is not APIHelper.SKIP:
            self.brand_name = brand_name 
        if locale is not APIHelper.SKIP:
            self.locale = locale 
        self.shipping_preference = shipping_preference 
        if payment_method is not APIHelper.SKIP:
            self.payment_method = payment_method 
        self.return_url = return_url 
        self.cancel_url = cancel_url 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        return_url = dictionary.get("return_url") if dictionary.get("return_url") else None
        cancel_url = dictionary.get("cancel_url") if dictionary.get("cancel_url") else None
        brand_name = dictionary.get("brand_name") if dictionary.get("brand_name") else APIHelper.SKIP
        locale = dictionary.get("locale") if dictionary.get("locale") else APIHelper.SKIP
        shipping_preference = dictionary.get("shipping_preference") if dictionary.get("shipping_preference") else 'GET_FROM_FILE'
        payment_method = PaymentMethod.from_dictionary(dictionary.get('payment_method')) if 'payment_method' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(return_url,
                   cancel_url,
                   brand_name,
                   locale,
                   shipping_preference,
                   payment_method)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'brand_name={(self.brand_name if hasattr(self, "brand_name") else None)!r}, '
                f'locale={(self.locale if hasattr(self, "locale") else None)!r}, '
                f'shipping_preference={(self.shipping_preference if hasattr(self, "shipping_preference") else None)!r}, '
                f'payment_method={(self.payment_method if hasattr(self, "payment_method") else None)!r}, '
                f'return_url={self.return_url!r}, '
                f'cancel_url={self.cancel_url!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'brand_name={(self.brand_name if hasattr(self, "brand_name") else None)!s}, '
                f'locale={(self.locale if hasattr(self, "locale") else None)!s}, '
                f'shipping_preference={(self.shipping_preference if hasattr(self, "shipping_preference") else None)!s}, '
                f'payment_method={(self.payment_method if hasattr(self, "payment_method") else None)!s}, '
                f'return_url={self.return_url!s}, '
                f'cancel_url={self.cancel_url!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ShippingType(object):

    """Implementation of the 'Shipping Type' enum.

    A classification for the method of purchase fulfillment.

    Attributes:
        SHIPPING: The payer intends to receive the items at a specified
            address.
        PICKUP: DEPRECATED. To ensure that seller protection is correctly
            assigned, please use 'PICKUP_IN_STORE' or 'PICKUP_FROM_PERSON'
            instead. Currently, this field indicates that the payer intends to
            pick up the items at a specified address (ie. a store address).
        PICKUP_IN_STORE: The payer intends to pick up the item(s) from the
            payee's physical store. Also termed as BOPIS, "Buy Online, Pick-up
            in Store". Seller protection is provided with this option.
        PICKUP_FROM_PERSON: The payer intends to pick up the item(s) from the
            payee in person. Also termed as BOPIP, "Buy Online, Pick-up in
            Person". Seller protection is not available, since the payer is
            receiving the item from the payee in person, and can validate the
            item prior to payment.

    """
    SHIPPING = 'SHIPPING'

    PICKUP = 'PICKUP'

    PICKUP_IN_STORE = 'PICKUP_IN_STORE'

    PICKUP_FROM_PERSON = 'PICKUP_FROM_PERSON'




__all__ = [
    'base_controller',
    'orders_controller',
    'payments_controller',
    'vault_controller',
    'transaction_search_controller',
    'subscriptions_controller',
    'o_auth_authorization_controller',
]



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaypalWalletAccountVerificationStatus(object):

    """Implementation of the 'PayPal Wallet Account Verification Status' enum.

    The account status indicates whether the buyer has verified the financial
    details associated with their PayPal account.

    Attributes:
        VERIFIED: The buyer has completed the verification of the financial
            details associated with this PayPal account. For example:
            confirming their bank account.
        UNVERIFIED: The buyer has not completed the verification of the
            financial details associated with this PayPal account. For
            example: confirming their bank account.

    """
    VERIFIED = 'VERIFIED'

    UNVERIFIED = 'UNVERIFIED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ApplePayPaymentDataType(object):

    """Implementation of the 'Apple Pay Payment Data Type' enum.

    Indicates the type of payment data passed, in case of Non China the
    payment data is 3DSECURE and for China it is EMV.

    Attributes:
        ENUM_3DSECURE: The card was authenticated using 3D Secure (3DS)
            authentication scheme. While using this value make sure to
            populate cryptogram and eci_indicator as part of payment data..
        EMV: The card was authenticated using EMV method, which is applicable
            for China. While using this value make sure to pass emv_data and
            pin as part of payment data.

    """
    ENUM_3DSECURE = '3DSECURE'

    EMV = 'EMV'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class SetupFeeFailureAction(object):

    """Implementation of the 'Setup Fee Failure Action' enum.

    The action to take on the subscription if the initial payment for the
    setup fails.

    Attributes:
        CONTINUE: Continues the subscription if the initial payment for the
            setup fails.
        CANCEL: Cancels the subscription if the initial payment for the setup
            fails.

    """
    CONTINUE = 'CONTINUE'

    CANCEL = 'CANCEL'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ReturnFlow(object):

    """Implementation of the 'Return Flow' enum.

    Merchant preference on how the buyer can navigate back to merchant website
    post approving the transaction on the Venmo App.

    Attributes:
        AUTO: After payment approval in the PayPal App, buyer will
            automatically be redirected to the merchant website.
        MANUAL: After payment approval in the PayPal App, buyer will be asked
            to manually navigate back to the merchant website where they
            started the transaction from. The buyer is shown a message like
            'Return to Merchant' to return to the source where the transaction
            actually started.

    """
    AUTO = 'AUTO'

    MANUAL = 'MANUAL'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ReasonCode(object):

    """Implementation of the 'Reason Code' enum.

    The reason code for the payment failure.

    Attributes:
        PAYMENT_DENIED: PayPal declined the payment due to one or more
            customer issues.
        INTERNAL_SERVER_ERROR: An internal server error has occurred.
        PAYEE_ACCOUNT_RESTRICTED: The payee account is not in good standing
            and cannot receive payments.
        PAYER_ACCOUNT_RESTRICTED: The payer account is not in good standing
            and cannot make payments.
        PAYER_CANNOT_PAY: Payer cannot pay for this transaction.
        SENDING_LIMIT_EXCEEDED: The transaction exceeds the payer's sending
            limit.
        TRANSACTION_RECEIVING_LIMIT_EXCEEDED: The transaction exceeds the
            receiver's receiving limit.
        CURRENCY_MISMATCH: The transaction is declined due to a currency
            mismatch.

    """
    PAYMENT_DENIED = 'PAYMENT_DENIED'

    INTERNAL_SERVER_ERROR = 'INTERNAL_SERVER_ERROR'

    PAYEE_ACCOUNT_RESTRICTED = 'PAYEE_ACCOUNT_RESTRICTED'

    PAYER_ACCOUNT_RESTRICTED = 'PAYER_ACCOUNT_RESTRICTED'

    PAYER_CANNOT_PAY = 'PAYER_CANNOT_PAY'

    SENDING_LIMIT_EXCEEDED = 'SENDING_LIMIT_EXCEEDED'

    TRANSACTION_RECEIVING_LIMIT_EXCEEDED = 'TRANSACTION_RECEIVING_LIMIT_EXCEEDED'

    CURRENCY_MISMATCH = 'CURRENCY_MISMATCH'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.balance_information import BalanceInformation


class BalancesResponse(object):

    """Implementation of the 'Balances Response' model.

    The balances response information.

    Attributes:
        balances (List[BalanceInformation]): An array of balance detail
            objects.
        account_id (str): The PayPal payer ID, which is a masked version of
            the PayPal account number intended for use with third parties. The
            account number is reversibly encrypted and a proprietary variant
            of Base32 is used to encode the result.
        as_of_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        last_refresh_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "balances": 'balances',
        "account_id": 'account_id',
        "as_of_time": 'as_of_time',
        "last_refresh_time": 'last_refresh_time'
    }

    _optionals = [
        'balances',
        'account_id',
        'as_of_time',
        'last_refresh_time',
    ]

    def __init__(self,
                 balances=APIHelper.SKIP,
                 account_id=APIHelper.SKIP,
                 as_of_time=APIHelper.SKIP,
                 last_refresh_time=APIHelper.SKIP):
        """Constructor for the BalancesResponse class"""

        # Initialize members of the class
        if balances is not APIHelper.SKIP:
            self.balances = balances 
        if account_id is not APIHelper.SKIP:
            self.account_id = account_id 
        if as_of_time is not APIHelper.SKIP:
            self.as_of_time = as_of_time 
        if last_refresh_time is not APIHelper.SKIP:
            self.last_refresh_time = last_refresh_time 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        balances = None
        if dictionary.get('balances') is not None:
            balances = [BalanceInformation.from_dictionary(x) for x in dictionary.get('balances')]
        else:
            balances = APIHelper.SKIP
        account_id = dictionary.get("account_id") if dictionary.get("account_id") else APIHelper.SKIP
        as_of_time = dictionary.get("as_of_time") if dictionary.get("as_of_time") else APIHelper.SKIP
        last_refresh_time = dictionary.get("last_refresh_time") if dictionary.get("last_refresh_time") else APIHelper.SKIP
        # Return an object of this model
        return cls(balances,
                   account_id,
                   as_of_time,
                   last_refresh_time)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'balances={(self.balances if hasattr(self, "balances") else None)!r}, '
                f'account_id={(self.account_id if hasattr(self, "account_id") else None)!r}, '
                f'as_of_time={(self.as_of_time if hasattr(self, "as_of_time") else None)!r}, '
                f'last_refresh_time={(self.last_refresh_time if hasattr(self, "last_refresh_time") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'balances={(self.balances if hasattr(self, "balances") else None)!s}, '
                f'account_id={(self.account_id if hasattr(self, "account_id") else None)!s}, '
                f'as_of_time={(self.as_of_time if hasattr(self, "as_of_time") else None)!s}, '
                f'last_refresh_time={(self.last_refresh_time if hasattr(self, "last_refresh_time") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CvvCode(object):

    """Implementation of the 'CVV Code' enum.

    The card verification value code for for Visa, Discover, Mastercard, or
    American Express.

    Attributes:
        CVV_E: For Visa, Mastercard, Discover, or American Express, error -
            unrecognized or unknown response.
        CVV_I: For Visa, Mastercard, Discover, or American Express, invalid or
            null.
        CVV_M: For Visa, Mastercard, Discover, or American Express, the
            CVV2/CSC matches.
        CVV_N: For Visa, Mastercard, Discover, or American Express, the
            CVV2/CSC does not match.
        CVV_P: For Visa, Mastercard, Discover, or American Express, it was not
            processed.
        CVV_S: For Visa, Mastercard, Discover, or American Express, the
            service is not supported.
        CVV_U: For Visa, Mastercard, Discover, or American Express, unknown -
            the issuer is not certified.
        CVV_X: For Visa, Mastercard, Discover, or American Express, no
            response. For Maestro, the service is not available.
        ENUM_ALL OTHERS: For Visa, Mastercard, Discover, or American Express,
            error.
        CVV_0: For Maestro, the CVV2 matched.
        CVV_1: For Maestro, the CVV2 did not match.
        CVV_2: For Maestro, the merchant has not implemented CVV2 code
            handling.
        CVV_3: For Maestro, the merchant has indicated that CVV2 is not
            present on card.
        CVV_4: For Maestro, the service is not available.

    """
    CVV_E = 'E'

    CVV_I = 'I'

    CVV_M = 'M'

    CVV_N = 'N'

    CVV_P = 'P'

    CVV_S = 'S'

    CVV_U = 'U'

    CVV_X = 'X'

    ENUM_ALL_OTHERS = 'All others'

    CVV_0 = '0'

    CVV_1 = '1'

    CVV_2 = '2'

    CVV_3 = '3'

    CVV_4 = '4'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CardVerificationStatus(object):

    """Implementation of the 'Card Verification Status' enum.

    Verification status of Card.

    Attributes:
        VERIFIED: Card has been verified
        FAILED: Card verification has failed

    """
    VERIFIED = 'VERIFIED'

    FAILED = 'FAILED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaypalWalletVaultStatus(object):

    """Implementation of the 'PayPal Wallet Vault Status' enum.

    The vault status.

    Attributes:
        VAULTED: The payment source has been saved in your customer's vault.
            This vault status reflects `/v3/vault` status.
        CREATED: DEPRECATED. The payment source has been saved in your
            customer's vault. This status applies to deprecated integration
            patterns and will not be returned for v3/vault integrations.
        APPROVED: Customer has approved the action of saving the specified
            payment_source into their vault. Use v3/vault/payment-tokens with
            given setup_token to save the payment source in the vault

    """
    VAULTED = 'VAULTED'

    CREATED = 'CREATED'

    APPROVED = 'APPROVED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class TaxesOverride(object):

    """Implementation of the 'Taxes Override' model.

    The tax details.

    Attributes:
        percentage (str): The percentage, as a fixed-point, signed decimal
            number. For example, define a 19.99% interest rate as `19.99`.
        inclusive (bool): Indicates whether the tax was already included in
            the billing amount.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "percentage": 'percentage',
        "inclusive": 'inclusive'
    }

    _optionals = [
        'percentage',
        'inclusive',
    ]

    def __init__(self,
                 percentage=APIHelper.SKIP,
                 inclusive=APIHelper.SKIP):
        """Constructor for the TaxesOverride class"""

        # Initialize members of the class
        if percentage is not APIHelper.SKIP:
            self.percentage = percentage 
        if inclusive is not APIHelper.SKIP:
            self.inclusive = inclusive 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        percentage = dictionary.get("percentage") if dictionary.get("percentage") else APIHelper.SKIP
        inclusive = dictionary.get("inclusive") if "inclusive" in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(percentage,
                   inclusive)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'percentage={(self.percentage if hasattr(self, "percentage") else None)!r}, '
                f'inclusive={(self.inclusive if hasattr(self, "inclusive") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'percentage={(self.percentage if hasattr(self, "percentage") else None)!s}, '
                f'inclusive={(self.inclusive if hasattr(self, "inclusive") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class IncentiveDetails(object):

    """Implementation of the 'Incentive Details' model.

    The incentive details.

    Attributes:
        incentive_type (str): The type of incentive, such as a special offer
            or coupon.
        incentive_code (str): The code that identifies an incentive, such as a
            coupon.
        incentive_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        incentive_program_code (str): The incentive program code that
            identifies a merchant loyalty or incentive program.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "incentive_type": 'incentive_type',
        "incentive_code": 'incentive_code',
        "incentive_amount": 'incentive_amount',
        "incentive_program_code": 'incentive_program_code'
    }

    _optionals = [
        'incentive_type',
        'incentive_code',
        'incentive_amount',
        'incentive_program_code',
    ]

    def __init__(self,
                 incentive_type=APIHelper.SKIP,
                 incentive_code=APIHelper.SKIP,
                 incentive_amount=APIHelper.SKIP,
                 incentive_program_code=APIHelper.SKIP):
        """Constructor for the IncentiveDetails class"""

        # Initialize members of the class
        if incentive_type is not APIHelper.SKIP:
            self.incentive_type = incentive_type 
        if incentive_code is not APIHelper.SKIP:
            self.incentive_code = incentive_code 
        if incentive_amount is not APIHelper.SKIP:
            self.incentive_amount = incentive_amount 
        if incentive_program_code is not APIHelper.SKIP:
            self.incentive_program_code = incentive_program_code 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        incentive_type = dictionary.get("incentive_type") if dictionary.get("incentive_type") else APIHelper.SKIP
        incentive_code = dictionary.get("incentive_code") if dictionary.get("incentive_code") else APIHelper.SKIP
        incentive_amount = Money.from_dictionary(dictionary.get('incentive_amount')) if 'incentive_amount' in dictionary.keys() else APIHelper.SKIP
        incentive_program_code = dictionary.get("incentive_program_code") if dictionary.get("incentive_program_code") else APIHelper.SKIP
        # Return an object of this model
        return cls(incentive_type,
                   incentive_code,
                   incentive_amount,
                   incentive_program_code)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'incentive_type={(self.incentive_type if hasattr(self, "incentive_type") else None)!r}, '
                f'incentive_code={(self.incentive_code if hasattr(self, "incentive_code") else None)!r}, '
                f'incentive_amount={(self.incentive_amount if hasattr(self, "incentive_amount") else None)!r}, '
                f'incentive_program_code={(self.incentive_program_code if hasattr(self, "incentive_program_code") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'incentive_type={(self.incentive_type if hasattr(self, "incentive_type") else None)!s}, '
                f'incentive_code={(self.incentive_code if hasattr(self, "incentive_code") else None)!s}, '
                f'incentive_amount={(self.incentive_amount if hasattr(self, "incentive_amount") else None)!s}, '
                f'incentive_program_code={(self.incentive_program_code if hasattr(self, "incentive_program_code") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money
from paypalserversdk.models.plan_override import PlanOverride
from paypalserversdk.models.subscriber_request import SubscriberRequest
from paypalserversdk.models.subscription_application_context import SubscriptionApplicationContext


class CreateSubscriptionRequest(object):

    """Implementation of the 'Create Subscription Request' model.

    The create subscription request details.

    Attributes:
        plan_id (str): The ID of the plan.
        start_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        quantity (str): The quantity of the product in the subscription.
        shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        subscriber (SubscriberRequest): The subscriber request information .
        auto_renewal (bool): DEPRECATED. Indicates whether the subscription
            auto-renews after the billing cycles complete.
        application_context (SubscriptionApplicationContext): DEPRECATED. The
            application context, which customizes the payer experience during
            the subscription approval process with PayPal.
        custom_id (str): The custom id for the subscription. Can be invoice id.
        plan (PlanOverride): An inline plan object to customise the
            subscription. You can override plan level default attributes by
            providing customised values for the subscription in this object.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "plan_id": 'plan_id',
        "start_time": 'start_time',
        "quantity": 'quantity',
        "shipping_amount": 'shipping_amount',
        "subscriber": 'subscriber',
        "auto_renewal": 'auto_renewal',
        "application_context": 'application_context',
        "custom_id": 'custom_id',
        "plan": 'plan'
    }

    _optionals = [
        'start_time',
        'quantity',
        'shipping_amount',
        'subscriber',
        'auto_renewal',
        'application_context',
        'custom_id',
        'plan',
    ]

    def __init__(self,
                 plan_id=None,
                 start_time=APIHelper.SKIP,
                 quantity=APIHelper.SKIP,
                 shipping_amount=APIHelper.SKIP,
                 subscriber=APIHelper.SKIP,
                 auto_renewal=False,
                 application_context=APIHelper.SKIP,
                 custom_id=APIHelper.SKIP,
                 plan=APIHelper.SKIP):
        """Constructor for the CreateSubscriptionRequest class"""

        # Initialize members of the class
        self.plan_id = plan_id 
        if start_time is not APIHelper.SKIP:
            self.start_time = start_time 
        if quantity is not APIHelper.SKIP:
            self.quantity = quantity 
        if shipping_amount is not APIHelper.SKIP:
            self.shipping_amount = shipping_amount 
        if subscriber is not APIHelper.SKIP:
            self.subscriber = subscriber 
        self.auto_renewal = auto_renewal 
        if application_context is not APIHelper.SKIP:
            self.application_context = application_context 
        if custom_id is not APIHelper.SKIP:
            self.custom_id = custom_id 
        if plan is not APIHelper.SKIP:
            self.plan = plan 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        plan_id = dictionary.get("plan_id") if dictionary.get("plan_id") else None
        start_time = dictionary.get("start_time") if dictionary.get("start_time") else APIHelper.SKIP
        quantity = dictionary.get("quantity") if dictionary.get("quantity") else APIHelper.SKIP
        shipping_amount = Money.from_dictionary(dictionary.get('shipping_amount')) if 'shipping_amount' in dictionary.keys() else APIHelper.SKIP
        subscriber = SubscriberRequest.from_dictionary(dictionary.get('subscriber')) if 'subscriber' in dictionary.keys() else APIHelper.SKIP
        auto_renewal = dictionary.get("auto_renewal") if dictionary.get("auto_renewal") else False
        application_context = SubscriptionApplicationContext.from_dictionary(dictionary.get('application_context')) if 'application_context' in dictionary.keys() else APIHelper.SKIP
        custom_id = dictionary.get("custom_id") if dictionary.get("custom_id") else APIHelper.SKIP
        plan = PlanOverride.from_dictionary(dictionary.get('plan')) if 'plan' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(plan_id,
                   start_time,
                   quantity,
                   shipping_amount,
                   subscriber,
                   auto_renewal,
                   application_context,
                   custom_id,
                   plan)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'plan_id={self.plan_id!r}, '
                f'start_time={(self.start_time if hasattr(self, "start_time") else None)!r}, '
                f'quantity={(self.quantity if hasattr(self, "quantity") else None)!r}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!r}, '
                f'subscriber={(self.subscriber if hasattr(self, "subscriber") else None)!r}, '
                f'auto_renewal={(self.auto_renewal if hasattr(self, "auto_renewal") else None)!r}, '
                f'application_context={(self.application_context if hasattr(self, "application_context") else None)!r}, '
                f'custom_id={(self.custom_id if hasattr(self, "custom_id") else None)!r}, '
                f'plan={(self.plan if hasattr(self, "plan") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'plan_id={self.plan_id!s}, '
                f'start_time={(self.start_time if hasattr(self, "start_time") else None)!s}, '
                f'quantity={(self.quantity if hasattr(self, "quantity") else None)!s}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!s}, '
                f'subscriber={(self.subscriber if hasattr(self, "subscriber") else None)!s}, '
                f'auto_renewal={(self.auto_renewal if hasattr(self, "auto_renewal") else None)!s}, '
                f'application_context={(self.application_context if hasattr(self, "application_context") else None)!s}, '
                f'custom_id={(self.custom_id if hasattr(self, "custom_id") else None)!s}, '
                f'plan={(self.plan if hasattr(self, "plan") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money
from paypalserversdk.models.pricing_tier import PricingTier


class SubscriptionPricingScheme(object):

    """Implementation of the 'Subscription Pricing Scheme' model.

    The pricing scheme details.

    Attributes:
        version (int): The version of the pricing scheme.
        fixed_price (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        pricing_model (SubscriptionPricingModel): The pricing model for tiered
            plan. The `tiers` parameter is required.
        tiers (List[PricingTier]): An array of pricing tiers which are used
            for billing volume/tiered plans. pricing_model field has to be
            specified.
        create_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        update_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "version": 'version',
        "fixed_price": 'fixed_price',
        "pricing_model": 'pricing_model',
        "tiers": 'tiers',
        "create_time": 'create_time',
        "update_time": 'update_time'
    }

    _optionals = [
        'version',
        'fixed_price',
        'pricing_model',
        'tiers',
        'create_time',
        'update_time',
    ]

    def __init__(self,
                 version=APIHelper.SKIP,
                 fixed_price=APIHelper.SKIP,
                 pricing_model=APIHelper.SKIP,
                 tiers=APIHelper.SKIP,
                 create_time=APIHelper.SKIP,
                 update_time=APIHelper.SKIP):
        """Constructor for the SubscriptionPricingScheme class"""

        # Initialize members of the class
        if version is not APIHelper.SKIP:
            self.version = version 
        if fixed_price is not APIHelper.SKIP:
            self.fixed_price = fixed_price 
        if pricing_model is not APIHelper.SKIP:
            self.pricing_model = pricing_model 
        if tiers is not APIHelper.SKIP:
            self.tiers = tiers 
        if create_time is not APIHelper.SKIP:
            self.create_time = create_time 
        if update_time is not APIHelper.SKIP:
            self.update_time = update_time 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        version = dictionary.get("version") if dictionary.get("version") else APIHelper.SKIP
        fixed_price = Money.from_dictionary(dictionary.get('fixed_price')) if 'fixed_price' in dictionary.keys() else APIHelper.SKIP
        pricing_model = dictionary.get("pricing_model") if dictionary.get("pricing_model") else APIHelper.SKIP
        tiers = None
        if dictionary.get('tiers') is not None:
            tiers = [PricingTier.from_dictionary(x) for x in dictionary.get('tiers')]
        else:
            tiers = APIHelper.SKIP
        create_time = dictionary.get("create_time") if dictionary.get("create_time") else APIHelper.SKIP
        update_time = dictionary.get("update_time") if dictionary.get("update_time") else APIHelper.SKIP
        # Return an object of this model
        return cls(version,
                   fixed_price,
                   pricing_model,
                   tiers,
                   create_time,
                   update_time)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'version={(self.version if hasattr(self, "version") else None)!r}, '
                f'fixed_price={(self.fixed_price if hasattr(self, "fixed_price") else None)!r}, '
                f'pricing_model={(self.pricing_model if hasattr(self, "pricing_model") else None)!r}, '
                f'tiers={(self.tiers if hasattr(self, "tiers") else None)!r}, '
                f'create_time={(self.create_time if hasattr(self, "create_time") else None)!r}, '
                f'update_time={(self.update_time if hasattr(self, "update_time") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'version={(self.version if hasattr(self, "version") else None)!s}, '
                f'fixed_price={(self.fixed_price if hasattr(self, "fixed_price") else None)!s}, '
                f'pricing_model={(self.pricing_model if hasattr(self, "pricing_model") else None)!s}, '
                f'tiers={(self.tiers if hasattr(self, "tiers") else None)!s}, '
                f'create_time={(self.create_time if hasattr(self, "create_time") else None)!s}, '
                f'update_time={(self.update_time if hasattr(self, "update_time") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class TokenType(object):

    """Implementation of the 'Token Type' enum.

    The tokenization method that generated the ID.

    Attributes:
        BILLING_AGREEMENT: The PayPal billing agreement ID. References an
            approved recurring payment for goods or services.

    """
    BILLING_AGREEMENT = 'BILLING_AGREEMENT'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class ParticipantMetadata(object):

    """Implementation of the 'Participant Metadata' model.

    Profile information of the sender or receiver.

    Attributes:
        ip_address (str): An Internet Protocol address (IP address). This
            address assigns a numerical label to each device that is connected
            to a computer network through the Internet Protocol. Supports IPv4
            and IPv6 addresses.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "ip_address": 'ip_address'
    }

    _optionals = [
        'ip_address',
    ]

    def __init__(self,
                 ip_address=APIHelper.SKIP):
        """Constructor for the ParticipantMetadata class"""

        # Initialize members of the class
        if ip_address is not APIHelper.SKIP:
            self.ip_address = ip_address 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        ip_address = dictionary.get("ip_address") if dictionary.get("ip_address") else APIHelper.SKIP
        # Return an object of this model
        return cls(ip_address)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'ip_address={(self.ip_address if hasattr(self, "ip_address") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'ip_address={(self.ip_address if hasattr(self, "ip_address") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaypalReferenceIdType(object):

    """Implementation of the 'PayPal Reference Id Type' enum.

    The PayPal reference ID type.

    Attributes:
        ODR: An order ID.
        TXN: A transaction ID.
        SUB: A subscription ID.
        PAP: A pre-approved payment ID.

    """
    ODR = 'ODR'

    TXN = 'TXN'

    SUB = 'SUB'

    PAP = 'PAP'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaypalExperienceLandingPage(object):

    """Implementation of the 'PayPal Experience Landing Page' enum.

    The type of landing page to show on the PayPal site for customer checkout.

    Attributes:
        LOGIN: When the customer clicks PayPal Checkout, the customer is
            redirected to a page to log in to PayPal and approve the payment.
        GUEST_CHECKOUT: When the customer clicks PayPal Checkout, the customer
            is redirected to a page to enter credit or debit card and other
            relevant billing information required to complete the purchase.
            This option has previously been also called as 'BILLING'
        NO_PREFERENCE: When the customer clicks PayPal Checkout, the customer
            is redirected to either a page to log in to PayPal and approve the
            payment or to a page to enter credit or debit card and other
            relevant billing information required to complete the purchase,
            depending on their previous interaction with PayPal.
        BILLING: DEPRECATED - please use GUEST_CHECKOUT. All implementations
            of 'BILLING' will be routed to 'GUEST_CHECKOUT'. When the customer
            clicks PayPal Checkout, the customer is redirected to a page to
            enter credit or debit card and other relevant billing information
            required to complete the purchase.

    """
    LOGIN = 'LOGIN'

    GUEST_CHECKOUT = 'GUEST_CHECKOUT'

    NO_PREFERENCE = 'NO_PREFERENCE'

    BILLING = 'BILLING'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaypalWalletContactPreference(object):

    """Implementation of the 'PayPal Wallet Contact Preference' enum.

    The preference to display the contact information (buyer’s shipping email
    & phone number) on PayPal's checkout for easy merchant-buyer communication.

    Attributes:
        NO_CONTACT_INFO: The merchant can opt out of showing buyer's contact
            information on PayPal checkout.
        UPDATE_CONTACT_INFO: The merchant allows buyer to add or update
            shipping contact information on the PayPal checkout. Please ensure
            to use this updated information returned in shipping.email_address
            and shipping.phone_number to contact your buyers.
        RETAIN_CONTACT_INFO: The buyer can only see but can not override
            merchant passed contact information (shipping.email_address and
            shipping.phone_number) on PayPal checkout. NOTE: If you don't pass
            the contact information, the behavior is the same as
            NO_CONTACT_INFO preference.

    """
    NO_CONTACT_INFO = 'NO_CONTACT_INFO'

    UPDATE_CONTACT_INFO = 'UPDATE_CONTACT_INFO'

    RETAIN_CONTACT_INFO = 'RETAIN_CONTACT_INFO'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CardType(object):

    """Implementation of the 'Card Type' enum.

    Type of card. i.e Credit, Debit and so on.

    Attributes:
        CREDIT: A credit card.
        DEBIT: A debit card.
        PREPAID: A Prepaid card.
        STORE: A store card.
        UNKNOWN: Card type cannot be determined.

    """
    CREDIT = 'CREDIT'

    DEBIT = 'DEBIT'

    PREPAID = 'PREPAID'

    STORE = 'STORE'

    UNKNOWN = 'UNKNOWN'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class TenureType(object):

    """Implementation of the 'Tenure Type' enum.

    The tenure type of the billing cycle identifies if the billing cycle is a
    trial(free or discounted) or regular billing cycle., The tenure type of
    the billing cycle. In case of a plan having trial cycle, only 2 trial
    cycles are allowed per plan., The type of the billing cycle.

    Attributes:
        REGULAR: A regular billing cycle to identify recurring charges for the
            billing agreement.
        TRIAL: A trial billing cycle to identify free or discounted charge for
            the billing agreement. Free trails will not have a price object in
            pricing scheme where as a discounted trial would have a discounted
            price compared to regular billing cycle.

    """
    REGULAR = 'REGULAR'

    TRIAL = 'TRIAL'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class ReauthorizeRequest(object):

    """Implementation of the 'Reauthorize Request' model.

    Reauthorizes an authorized PayPal account payment, by ID. To ensure that
    funds are still available, reauthorize a payment after its initial
    three-day honor period expires. You can reauthorize a payment only once
    from days four to 29. If 30 days have transpired since the date of the
    original authorization, you must create an authorized payment instead of
    reauthorizing the original authorized payment. A reauthorized payment
    itself has a new honor period of three days. You can reauthorize an
    authorized payment once. The allowed amount depends on context and
    geography, for example in US it is up to 115% of the original authorized
    amount, not to exceed an increase of $75 USD. Supports only the `amount`
    request parameter.

    Attributes:
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount": 'amount'
    }

    _optionals = [
        'amount',
    ]

    def __init__(self,
                 amount=APIHelper.SKIP):
        """Constructor for the ReauthorizeRequest class"""

        # Initialize members of the class
        if amount is not APIHelper.SKIP:
            self.amount = amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = Money.from_dictionary(dictionary.get('amount')) if 'amount' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(amount)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'amount={(self.amount if hasattr(self, "amount") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'amount={(self.amount if hasattr(self, "amount") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.customer_response import CustomerResponse
from paypalserversdk.models.link_description import LinkDescription
from paypalserversdk.models.payment_token_response_payment_source import PaymentTokenResponsePaymentSource


class PaymentTokenResponse(object):

    """Implementation of the 'Payment Token Response' model.

    Full representation of a saved payment token.

    Attributes:
        id (str): The PayPal-generated ID for the vaulted payment source. This
            ID should be stored on the merchant's server so the saved payment
            source can be used for future transactions.
        customer (CustomerResponse): Customer in merchant's or partner's
            system of records.
        payment_source (PaymentTokenResponsePaymentSource): The vaulted
            payment method details.
        links (List[LinkDescription]): An array of related [HATEOAS
            links](/api/rest/responses/#hateoas).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "customer": 'customer',
        "payment_source": 'payment_source',
        "links": 'links'
    }

    _optionals = [
        'id',
        'customer',
        'payment_source',
        'links',
    ]

    def __init__(self,
                 id=APIHelper.SKIP,
                 customer=APIHelper.SKIP,
                 payment_source=APIHelper.SKIP,
                 links=APIHelper.SKIP):
        """Constructor for the PaymentTokenResponse class"""

        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id 
        if customer is not APIHelper.SKIP:
            self.customer = customer 
        if payment_source is not APIHelper.SKIP:
            self.payment_source = payment_source 
        if links is not APIHelper.SKIP:
            self.links = links 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        customer = CustomerResponse.from_dictionary(dictionary.get('customer')) if 'customer' in dictionary.keys() else APIHelper.SKIP
        payment_source = PaymentTokenResponsePaymentSource.from_dictionary(dictionary.get('payment_source')) if 'payment_source' in dictionary.keys() else APIHelper.SKIP
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   customer,
                   payment_source,
                   links)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'customer={(self.customer if hasattr(self, "customer") else None)!r}, '
                f'payment_source={(self.payment_source if hasattr(self, "payment_source") else None)!r}, '
                f'links={(self.links if hasattr(self, "links") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'customer={(self.customer if hasattr(self, "customer") else None)!s}, '
                f'payment_source={(self.payment_source if hasattr(self, "payment_source") else None)!s}, '
                f'links={(self.links if hasattr(self, "links") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.checkout_option import CheckoutOption
from paypalserversdk.models.money import Money
from paypalserversdk.models.tax_amount import TaxAmount


class ItemDetails(object):

    """Implementation of the 'Item Details' model.

    The item details.

    Attributes:
        item_code (str): An item code that identifies a merchant's goods or
            service.
        item_name (str): The item name.
        item_description (str): The item description.
        item_options (str): The item options. Describes option choices on the
            purchase of the item in some detail.
        item_quantity (str): The number of purchased units of goods or a
            service.
        item_unit_price (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        item_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        discount_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        adjustment_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        gift_wrap_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        tax_percentage (str): The percentage, as a fixed-point, signed decimal
            number. For example, define a 19.99% interest rate as `19.99`.
        tax_amounts (List[TaxAmount]): An array of tax amounts levied by a
            government on the purchase of goods or services.
        basic_shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        extra_shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        handling_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        insurance_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        total_item_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        invoice_number (str): The invoice number. An alphanumeric string that
            identifies a billing for a merchant.
        checkout_options (List[CheckoutOption]): An array of checkout options.
            Each option has a name and value.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "item_code": 'item_code',
        "item_name": 'item_name',
        "item_description": 'item_description',
        "item_options": 'item_options',
        "item_quantity": 'item_quantity',
        "item_unit_price": 'item_unit_price',
        "item_amount": 'item_amount',
        "discount_amount": 'discount_amount',
        "adjustment_amount": 'adjustment_amount',
        "gift_wrap_amount": 'gift_wrap_amount',
        "tax_percentage": 'tax_percentage',
        "tax_amounts": 'tax_amounts',
        "basic_shipping_amount": 'basic_shipping_amount',
        "extra_shipping_amount": 'extra_shipping_amount',
        "handling_amount": 'handling_amount',
        "insurance_amount": 'insurance_amount',
        "total_item_amount": 'total_item_amount',
        "invoice_number": 'invoice_number',
        "checkout_options": 'checkout_options'
    }

    _optionals = [
        'item_code',
        'item_name',
        'item_description',
        'item_options',
        'item_quantity',
        'item_unit_price',
        'item_amount',
        'discount_amount',
        'adjustment_amount',
        'gift_wrap_amount',
        'tax_percentage',
        'tax_amounts',
        'basic_shipping_amount',
        'extra_shipping_amount',
        'handling_amount',
        'insurance_amount',
        'total_item_amount',
        'invoice_number',
        'checkout_options',
    ]

    def __init__(self,
                 item_code=APIHelper.SKIP,
                 item_name=APIHelper.SKIP,
                 item_description=APIHelper.SKIP,
                 item_options=APIHelper.SKIP,
                 item_quantity=APIHelper.SKIP,
                 item_unit_price=APIHelper.SKIP,
                 item_amount=APIHelper.SKIP,
                 discount_amount=APIHelper.SKIP,
                 adjustment_amount=APIHelper.SKIP,
                 gift_wrap_amount=APIHelper.SKIP,
                 tax_percentage=APIHelper.SKIP,
                 tax_amounts=APIHelper.SKIP,
                 basic_shipping_amount=APIHelper.SKIP,
                 extra_shipping_amount=APIHelper.SKIP,
                 handling_amount=APIHelper.SKIP,
                 insurance_amount=APIHelper.SKIP,
                 total_item_amount=APIHelper.SKIP,
                 invoice_number=APIHelper.SKIP,
                 checkout_options=APIHelper.SKIP):
        """Constructor for the ItemDetails class"""

        # Initialize members of the class
        if item_code is not APIHelper.SKIP:
            self.item_code = item_code 
        if item_name is not APIHelper.SKIP:
            self.item_name = item_name 
        if item_description is not APIHelper.SKIP:
            self.item_description = item_description 
        if item_options is not APIHelper.SKIP:
            self.item_options = item_options 
        if item_quantity is not APIHelper.SKIP:
            self.item_quantity = item_quantity 
        if item_unit_price is not APIHelper.SKIP:
            self.item_unit_price = item_unit_price 
        if item_amount is not APIHelper.SKIP:
            self.item_amount = item_amount 
        if discount_amount is not APIHelper.SKIP:
            self.discount_amount = discount_amount 
        if adjustment_amount is not APIHelper.SKIP:
            self.adjustment_amount = adjustment_amount 
        if gift_wrap_amount is not APIHelper.SKIP:
            self.gift_wrap_amount = gift_wrap_amount 
        if tax_percentage is not APIHelper.SKIP:
            self.tax_percentage = tax_percentage 
        if tax_amounts is not APIHelper.SKIP:
            self.tax_amounts = tax_amounts 
        if basic_shipping_amount is not APIHelper.SKIP:
            self.basic_shipping_amount = basic_shipping_amount 
        if extra_shipping_amount is not APIHelper.SKIP:
            self.extra_shipping_amount = extra_shipping_amount 
        if handling_amount is not APIHelper.SKIP:
            self.handling_amount = handling_amount 
        if insurance_amount is not APIHelper.SKIP:
            self.insurance_amount = insurance_amount 
        if total_item_amount is not APIHelper.SKIP:
            self.total_item_amount = total_item_amount 
        if invoice_number is not APIHelper.SKIP:
            self.invoice_number = invoice_number 
        if checkout_options is not APIHelper.SKIP:
            self.checkout_options = checkout_options 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        item_code = dictionary.get("item_code") if dictionary.get("item_code") else APIHelper.SKIP
        item_name = dictionary.get("item_name") if dictionary.get("item_name") else APIHelper.SKIP
        item_description = dictionary.get("item_description") if dictionary.get("item_description") else APIHelper.SKIP
        item_options = dictionary.get("item_options") if dictionary.get("item_options") else APIHelper.SKIP
        item_quantity = dictionary.get("item_quantity") if dictionary.get("item_quantity") else APIHelper.SKIP
        item_unit_price = Money.from_dictionary(dictionary.get('item_unit_price')) if 'item_unit_price' in dictionary.keys() else APIHelper.SKIP
        item_amount = Money.from_dictionary(dictionary.get('item_amount')) if 'item_amount' in dictionary.keys() else APIHelper.SKIP
        discount_amount = Money.from_dictionary(dictionary.get('discount_amount')) if 'discount_amount' in dictionary.keys() else APIHelper.SKIP
        adjustment_amount = Money.from_dictionary(dictionary.get('adjustment_amount')) if 'adjustment_amount' in dictionary.keys() else APIHelper.SKIP
        gift_wrap_amount = Money.from_dictionary(dictionary.get('gift_wrap_amount')) if 'gift_wrap_amount' in dictionary.keys() else APIHelper.SKIP
        tax_percentage = dictionary.get("tax_percentage") if dictionary.get("tax_percentage") else APIHelper.SKIP
        tax_amounts = None
        if dictionary.get('tax_amounts') is not None:
            tax_amounts = [TaxAmount.from_dictionary(x) for x in dictionary.get('tax_amounts')]
        else:
            tax_amounts = APIHelper.SKIP
        basic_shipping_amount = Money.from_dictionary(dictionary.get('basic_shipping_amount')) if 'basic_shipping_amount' in dictionary.keys() else APIHelper.SKIP
        extra_shipping_amount = Money.from_dictionary(dictionary.get('extra_shipping_amount')) if 'extra_shipping_amount' in dictionary.keys() else APIHelper.SKIP
        handling_amount = Money.from_dictionary(dictionary.get('handling_amount')) if 'handling_amount' in dictionary.keys() else APIHelper.SKIP
        insurance_amount = Money.from_dictionary(dictionary.get('insurance_amount')) if 'insurance_amount' in dictionary.keys() else APIHelper.SKIP
        total_item_amount = Money.from_dictionary(dictionary.get('total_item_amount')) if 'total_item_amount' in dictionary.keys() else APIHelper.SKIP
        invoice_number = dictionary.get("invoice_number") if dictionary.get("invoice_number") else APIHelper.SKIP
        checkout_options = None
        if dictionary.get('checkout_options') is not None:
            checkout_options = [CheckoutOption.from_dictionary(x) for x in dictionary.get('checkout_options')]
        else:
            checkout_options = APIHelper.SKIP
        # Return an object of this model
        return cls(item_code,
                   item_name,
                   item_description,
                   item_options,
                   item_quantity,
                   item_unit_price,
                   item_amount,
                   discount_amount,
                   adjustment_amount,
                   gift_wrap_amount,
                   tax_percentage,
                   tax_amounts,
                   basic_shipping_amount,
                   extra_shipping_amount,
                   handling_amount,
                   insurance_amount,
                   total_item_amount,
                   invoice_number,
                   checkout_options)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'item_code={(self.item_code if hasattr(self, "item_code") else None)!r}, '
                f'item_name={(self.item_name if hasattr(self, "item_name") else None)!r}, '
                f'item_description={(self.item_description if hasattr(self, "item_description") else None)!r}, '
                f'item_options={(self.item_options if hasattr(self, "item_options") else None)!r}, '
                f'item_quantity={(self.item_quantity if hasattr(self, "item_quantity") else None)!r}, '
                f'item_unit_price={(self.item_unit_price if hasattr(self, "item_unit_price") else None)!r}, '
                f'item_amount={(self.item_amount if hasattr(self, "item_amount") else None)!r}, '
                f'discount_amount={(self.discount_amount if hasattr(self, "discount_amount") else None)!r}, '
                f'adjustment_amount={(self.adjustment_amount if hasattr(self, "adjustment_amount") else None)!r}, '
                f'gift_wrap_amount={(self.gift_wrap_amount if hasattr(self, "gift_wrap_amount") else None)!r}, '
                f'tax_percentage={(self.tax_percentage if hasattr(self, "tax_percentage") else None)!r}, '
                f'tax_amounts={(self.tax_amounts if hasattr(self, "tax_amounts") else None)!r}, '
                f'basic_shipping_amount={(self.basic_shipping_amount if hasattr(self, "basic_shipping_amount") else None)!r}, '
                f'extra_shipping_amount={(self.extra_shipping_amount if hasattr(self, "extra_shipping_amount") else None)!r}, '
                f'handling_amount={(self.handling_amount if hasattr(self, "handling_amount") else None)!r}, '
                f'insurance_amount={(self.insurance_amount if hasattr(self, "insurance_amount") else None)!r}, '
                f'total_item_amount={(self.total_item_amount if hasattr(self, "total_item_amount") else None)!r}, '
                f'invoice_number={(self.invoice_number if hasattr(self, "invoice_number") else None)!r}, '
                f'checkout_options={(self.checkout_options if hasattr(self, "checkout_options") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'item_code={(self.item_code if hasattr(self, "item_code") else None)!s}, '
                f'item_name={(self.item_name if hasattr(self, "item_name") else None)!s}, '
                f'item_description={(self.item_description if hasattr(self, "item_description") else None)!s}, '
                f'item_options={(self.item_options if hasattr(self, "item_options") else None)!s}, '
                f'item_quantity={(self.item_quantity if hasattr(self, "item_quantity") else None)!s}, '
                f'item_unit_price={(self.item_unit_price if hasattr(self, "item_unit_price") else None)!s}, '
                f'item_amount={(self.item_amount if hasattr(self, "item_amount") else None)!s}, '
                f'discount_amount={(self.discount_amount if hasattr(self, "discount_amount") else None)!s}, '
                f'adjustment_amount={(self.adjustment_amount if hasattr(self, "adjustment_amount") else None)!s}, '
                f'gift_wrap_amount={(self.gift_wrap_amount if hasattr(self, "gift_wrap_amount") else None)!s}, '
                f'tax_percentage={(self.tax_percentage if hasattr(self, "tax_percentage") else None)!s}, '
                f'tax_amounts={(self.tax_amounts if hasattr(self, "tax_amounts") else None)!s}, '
                f'basic_shipping_amount={(self.basic_shipping_amount if hasattr(self, "basic_shipping_amount") else None)!s}, '
                f'extra_shipping_amount={(self.extra_shipping_amount if hasattr(self, "extra_shipping_amount") else None)!s}, '
                f'handling_amount={(self.handling_amount if hasattr(self, "handling_amount") else None)!s}, '
                f'insurance_amount={(self.insurance_amount if hasattr(self, "insurance_amount") else None)!s}, '
                f'total_item_amount={(self.total_item_amount if hasattr(self, "total_item_amount") else None)!s}, '
                f'invoice_number={(self.invoice_number if hasattr(self, "invoice_number") else None)!s}, '
                f'checkout_options={(self.checkout_options if hasattr(self, "checkout_options") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money
from paypalserversdk.models.order_billing_plan import OrderBillingPlan
from paypalserversdk.models.universal_product_code import UniversalProductCode


class ItemRequest(object):

    """Implementation of the 'Item Request' model.

    The details for the items to be purchased.

    Attributes:
        name (str): The item name or title.
        unit_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        tax (Money): The currency and amount for a financial transaction, such
            as a balance or payment due.
        quantity (str): The item quantity. Must be a whole number.
        description (str): This field supports up to 4000 characters, but any
            content beyond 2048 characters (including spaces) will be
            truncated. The 2048 character limit is reflected in the response
            representation of this field.
        sku (str): The stock keeping unit (SKU) for the item.
        url (str): The URL to the item being purchased. Visible to buyer and
            used in buyer experiences.
        category (ItemCategory): The item category type.
        image_url (str): The URL of the item's image. File type and size
            restrictions apply. An image that violates these restrictions will
            not be honored.
        upc (UniversalProductCode): The Universal Product Code of the item.
        billing_plan (OrderBillingPlan): Metadata for merchant-managed
            recurring billing plans. Valid only during the saved payment
            method token or billing agreement creation.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "unit_amount": 'unit_amount',
        "quantity": 'quantity',
        "tax": 'tax',
        "description": 'description',
        "sku": 'sku',
        "url": 'url',
        "category": 'category',
        "image_url": 'image_url',
        "upc": 'upc',
        "billing_plan": 'billing_plan'
    }

    _optionals = [
        'tax',
        'description',
        'sku',
        'url',
        'category',
        'image_url',
        'upc',
        'billing_plan',
    ]

    def __init__(self,
                 name=None,
                 unit_amount=None,
                 quantity=None,
                 tax=APIHelper.SKIP,
                 description=APIHelper.SKIP,
                 sku=APIHelper.SKIP,
                 url=APIHelper.SKIP,
                 category=APIHelper.SKIP,
                 image_url=APIHelper.SKIP,
                 upc=APIHelper.SKIP,
                 billing_plan=APIHelper.SKIP):
        """Constructor for the ItemRequest class"""

        # Initialize members of the class
        self.name = name 
        self.unit_amount = unit_amount 
        if tax is not APIHelper.SKIP:
            self.tax = tax 
        self.quantity = quantity 
        if description is not APIHelper.SKIP:
            self.description = description 
        if sku is not APIHelper.SKIP:
            self.sku = sku 
        if url is not APIHelper.SKIP:
            self.url = url 
        if category is not APIHelper.SKIP:
            self.category = category 
        if image_url is not APIHelper.SKIP:
            self.image_url = image_url 
        if upc is not APIHelper.SKIP:
            self.upc = upc 
        if billing_plan is not APIHelper.SKIP:
            self.billing_plan = billing_plan 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else None
        unit_amount = Money.from_dictionary(dictionary.get('unit_amount')) if dictionary.get('unit_amount') else None
        quantity = dictionary.get("quantity") if dictionary.get("quantity") else None
        tax = Money.from_dictionary(dictionary.get('tax')) if 'tax' in dictionary.keys() else APIHelper.SKIP
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        sku = dictionary.get("sku") if dictionary.get("sku") else APIHelper.SKIP
        url = dictionary.get("url") if dictionary.get("url") else APIHelper.SKIP
        category = dictionary.get("category") if dictionary.get("category") else APIHelper.SKIP
        image_url = dictionary.get("image_url") if dictionary.get("image_url") else APIHelper.SKIP
        upc = UniversalProductCode.from_dictionary(dictionary.get('upc')) if 'upc' in dictionary.keys() else APIHelper.SKIP
        billing_plan = OrderBillingPlan.from_dictionary(dictionary.get('billing_plan')) if 'billing_plan' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   unit_amount,
                   quantity,
                   tax,
                   description,
                   sku,
                   url,
                   category,
                   image_url,
                   upc,
                   billing_plan)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'name={self.name!r}, '
                f'unit_amount={self.unit_amount!r}, '
                f'tax={(self.tax if hasattr(self, "tax") else None)!r}, '
                f'quantity={self.quantity!r}, '
                f'description={(self.description if hasattr(self, "description") else None)!r}, '
                f'sku={(self.sku if hasattr(self, "sku") else None)!r}, '
                f'url={(self.url if hasattr(self, "url") else None)!r}, '
                f'category={(self.category if hasattr(self, "category") else None)!r}, '
                f'image_url={(self.image_url if hasattr(self, "image_url") else None)!r}, '
                f'upc={(self.upc if hasattr(self, "upc") else None)!r}, '
                f'billing_plan={(self.billing_plan if hasattr(self, "billing_plan") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'name={self.name!s}, '
                f'unit_amount={self.unit_amount!s}, '
                f'tax={(self.tax if hasattr(self, "tax") else None)!s}, '
                f'quantity={self.quantity!s}, '
                f'description={(self.description if hasattr(self, "description") else None)!s}, '
                f'sku={(self.sku if hasattr(self, "sku") else None)!s}, '
                f'url={(self.url if hasattr(self, "url") else None)!s}, '
                f'category={(self.category if hasattr(self, "category") else None)!s}, '
                f'image_url={(self.image_url if hasattr(self, "image_url") else None)!s}, '
                f'upc={(self.upc if hasattr(self, "upc") else None)!s}, '
                f'billing_plan={(self.billing_plan if hasattr(self, "billing_plan") else None)!s})')



__all__ = [
    'activity_timestamps',
    'amount_breakdown',
    'amount_with_breakdown',
    'app_switch_context',
    'apple_pay_payment_object',
    'apple_pay_attributes',
    'apple_pay_attributes_response',
    'apple_pay_card_response',
    'apple_pay_decrypted_token_data',
    'apple_pay_experience_context',
    'apple_pay_payment_data',
    'apple_pay_request',
    'assurance_details',
    'authentication_response',
    'authorization',
    'link_description',
    'authorization_status_with_details',
    'authorization_status_details',
    'authorization_with_additional_data',
    'bancontact_payment_object',
    'bancontact_payment_request',
    'billing_cycle',
    'bin_details',
    'blik_payment_object',
    'blik_experience_context',
    'blik_one_click_payment_request',
    'blik_one_click_payment_object',
    'blik_payment_request',
    'blik_level_0_payment_object',
    'callback_configuration',
    'orders_capture',
    'capture_status_with_details',
    'capture_status_details',
    'card_attributes',
    'card_attributes_response',
    'card_customer_information',
    'card_experience_context',
    'card_from_request',
    'card_request',
    'card_response',
    'card_stored_credential',
    'card_supplementary_data',
    'card_vault_response',
    'card_verification',
    'cobranded_card',
    'confirm_order_request',
    'customer_information',
    'platform_fee',
    'money',
    'eps_payment_object',
    'eps_payment_request',
    'error_details',
    'exchange_rate',
    'experience_context',
    'giropay_payment_object',
    'giropay_payment_request',
    'google_pay_wallet_response',
    'google_pay_card_response',
    'google_pay_decrypted_token_data',
    'google_pay_experience_context',
    'google_pay_request',
    'ideal_payment_object',
    'ideal_payment_request',
    'item',
    'item_request',
    'level_2_card_processing_data',
    'level_3_card_processing_data',
    'line_item',
    'mobile_web_context',
    'mybank_payment_object',
    'mybank_payment_request',
    'native_app_context',
    'net_amount_breakdown_item',
    'network_token',
    'network_transaction',
    'order',
    'order_application_context',
    'order_authorize_request',
    'order_authorize_response',
    'order_billing_plan',
    'order_capture_request',
    'order_confirm_application_context',
    'order_request',
    'order_tracker_request',
    'p_24_payment_object',
    'p_24_payment_request',
    'participant_metadata',
    'patch',
    'payee_base',
    'payer',
    'payment_collection',
    'payment_instruction',
    'payment_method_preference',
    'payment_source',
    'payment_source_response',
    'paypal_wallet',
    'paypal_wallet_attributes',
    'paypal_wallet_attributes_response',
    'paypal_wallet_customer',
    'paypal_wallet_experience_context',
    'paypal_wallet_response',
    'paypal_wallet_stored_credential',
    'paypal_wallet_vault_response',
    'phone_number',
    'phone_with_type',
    'pricing_scheme',
    'processor_response',
    'purchase_unit',
    'purchase_unit_request',
    'refund',
    'refund_status_with_details',
    'refund_status_details',
    'risk_supplementary_data',
    'seller_payable_breakdown',
    'seller_protection',
    'seller_receivable_breakdown',
    'shipping_details',
    'shipping_option',
    'sofort_payment_object',
    'sofort_payment_request',
    'stored_payment_source',
    'supplementary_data',
    'tax_info',
    'three_d_secure_authentication_response',
    'token',
    'order_tracker_response',
    'order_tracker_item',
    'trustly_payment_object',
    'trustly_payment_request',
    'universal_product_code',
    'vault_instruction',
    'vault_instruction_base',
    'vault_response',
    'venmo_wallet_vault_attributes',
    'venmo_vault_response',
    'venmo_wallet_additional_attributes',
    'venmo_wallet_attributes_response',
    'venmo_wallet_experience_context',
    'venmo_wallet_request',
    'venmo_wallet_response',
    'payer_base',
    'address',
    'apple_pay_tokenized_card',
    'google_pay_card',
    'google_pay_request_card',
    'name',
    'order_authorize_request_payment_source',
    'order_authorize_response_payment_source',
    'order_capture_request_payment_source',
    'paypal_wallet_customer_request',
    'paypal_wallet_vault_instruction',
    'phone_number_with_country_code',
    'phone_number_with_optional_country_code',
    'shipping_name',
    'shipping_with_tracking_details',
    'vault_customer',
    'venmo_wallet_customer_information',
    'payment_authorization',
    'payments_capture',
    'captured_payment',
    'capture_request',
    'refund_platform_fee',
    'refund_payment_instruction',
    'reauthorize_request',
    'refund_request',
    'related_identifiers',
    'payment_supplementary_data',
    'capture_payment_instruction',
    'apple_pay_payment_token',
    'vault_apple_pay_request',
    'bank_request',
    'vault_card_experience_context',
    'card_payment_token_entity',
    'card_verification_details',
    'customer',
    'customer_vault_payment_tokens_response',
    'payment_token_request_payment_source',
    'vault_experience_context',
    'network_transaction_reference_entity',
    'one_time_charge',
    'payment_token_response_payment_source',
    'payment_token_request',
    'payment_token_response',
    'vault_paypal_wallet_request',
    'paypal_payment_token',
    'phone',
    'plan',
    'sepa_debit_experience_context',
    'sepa_debit_request',
    'setup_token_request',
    'setup_token_response',
    'three_d_secure_card_authentication_response',
    'vault_token_request',
    'venmo_experience_context',
    'vault_venmo_request',
    'venmo_payment_token',
    'vaulted_digital_wallet',
    'apple_pay_card',
    'apple_pay_request_card',
    'card_authentication_response',
    'card_response_address',
    'card_verification_processor_response',
    'customer_response',
    'payment_token_request_card',
    'setup_token_request_card',
    'setup_token_request_payment_source',
    'setup_token_response_card',
    'setup_token_response_payment_source',
    'vault_response_customer',
    'vaulted_digital_wallet_shipping_details',
    'simple_postal_address_coarse_grained',
    'auction_information',
    'balance_information',
    'balances_response',
    'cart_information',
    'checkout_option',
    'transaction_search_error_details',
    'incentive_details',
    'incentive_information',
    'item_details',
    'tax_amount',
    'payer_name',
    'payer_information',
    'search_response',
    'shipping_information',
    'store_information',
    'transaction_details',
    'transaction_information',
    'subscription_amount_with_breakdown',
    'subscription_application_context',
    'subscription_billing_cycle',
    'billing_cycle_override',
    'subscriptions_card_attributes',
    'card_customer',
    'subscription_card_request',
    'card_response_with_billing_address',
    'subscription_customer_information',
    'cycle_execution',
    'failed_payment_details',
    'frequency',
    'merchant_preferences',
    'subscription_payer_name',
    'payment_preferences',
    'payment_preferences_override',
    'subscription_payment_source',
    'subscription_payment_source_response',
    'billing_plan',
    'plan_collection',
    'plan_override',
    'plan_request',
    'subscription_pricing_scheme',
    'pricing_tier',
    'subscriber',
    'subscriber_request',
    'subscription',
    'activate_subscription_request',
    'subscription_billing_information',
    'cancel_subscription_request',
    'capture_subscription_request',
    'subscription_collection',
    'create_subscription_request',
    'modify_subscription_request',
    'modify_subscription_response',
    'suspend_subscription',
    'taxes',
    'taxes_override',
    'subscription_transaction_details',
    'transactions_list',
    'update_pricing_scheme',
    'update_pricing_schemes_request',
    'last_payment_details',
    'payment_method',
    'plan_details',
    'subscription_patch_application_context',
    'o_auth_token',
    'callback_events',
    'card_brand',
    'card_type',
    'usage_pattern',
    'checkout_payment_intent',
    'disbursement_mode',
    'dispute_category',
    'eci_flag',
    'enrollment_status',
    'liability_shift_indicator',
    'order_status',
    'pa_res_status',
    'payee_payment_method_preference',
    'payment_initiator',
    'phone_type',
    'shipment_carrier',
    'shipping_type',
    'store_in_vault_instruction',
    'stored_payment_source_payment_type',
    'stored_payment_source_usage_type',
    'order_tracker_status',
    'apple_pay_payment_data_type',
    'authorization_incomplete_reason',
    'authorization_status',
    'avs_code',
    'capture_incomplete_reason',
    'capture_status',
    'cvv_code',
    'experience_context_shipping_preference',
    'experience_status',
    'fulfillment_type',
    'google_pay_authentication_method',
    'google_pay_payment_method',
    'item_category',
    'link_http_method',
    'mobile_return_flow',
    'order_application_context_landing_page',
    'order_application_context_shipping_preference',
    'order_application_context_user_action',
    'orders_card_verification_method',
    'os_type',
    'patch_op',
    'payment_advice_code',
    'paypal_experience_landing_page',
    'paypal_experience_user_action',
    'paypal_payment_token_customer_type',
    'paypal_payment_token_usage_type',
    'paypal_wallet_account_verification_status',
    'paypal_wallet_contact_preference',
    'paypal_wallet_context_shipping_preference',
    'paypal_wallet_vault_status',
    'pricing_model',
    'processor_response_code',
    'refund_incomplete_reason',
    'refund_status',
    'return_flow',
    'seller_protection_status',
    'standard_entry_class_code',
    'tax_id_type',
    'tenure_type',
    'token_type',
    'upc_type',
    'vault_status',
    'venmo_payment_token_customer_type',
    'venmo_payment_token_usage_pattern',
    'venmo_payment_token_usage_type',
    'venmo_vault_response_status',
    'venmo_wallet_experience_context_shipping_preference',
    'venmo_wallet_experience_context_user_action',
    'vault_card_verification_method',
    'card_verification_status',
    'payment_token_status',
    'vault_user_action',
    'vault_instruction_action',
    'vault_token_request_type',
    'paypal_reference_id_type',
    'subscriptions_card_brand',
    'application_context_user_action',
    'capture_type',
    'interval_unit',
    'plan_request_status',
    'reason_code',
    'setup_fee_failure_action',
    'subscription_plan_status',
    'subscription_pricing_model',
    'o_auth_provider_error',
]



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PatchOp(object):

    """Implementation of the 'Patch Op' enum.

    The operation.

    Attributes:
        ADD: Depending on the target location reference, completes one of
            these functions: The target location is an array index. Inserts a
            new value into the array at the specified index. The target
            location is an object parameter that does not already exist. Adds
            a new parameter to the object. The target location is an object
            parameter that does exist. Replaces that parameter's value. The
            value parameter defines the value to add. For more information,
            see 4.1. add.
        REMOVE: Removes the value at the target location. For the operation to
            succeed, the target location must exist. For more information, see
            4.2. remove.
        REPLACE: Replaces the value at the target location with a new value.
            The operation object must contain a value parameter that defines
            the replacement value. For the operation to succeed, the target
            location must exist. For more information, see 4.3. replace.
        MOVE: Removes the value at a specified location and adds it to the
            target location. The operation object must contain a from
            parameter, which is a string that contains a JSON pointer value
            that references the location in the target document from which to
            move the value. For the operation to succeed, the from location
            must exist. For more information, see 4.4. move.
        COPY: Copies the value at a specified location to the target location.
            The operation object must contain a from parameter, which is a
            string that contains a JSON pointer value that references the
            location in the target document from which to copy the value. For
            the operation to succeed, the from location must exist. For more
            information, see 4.5. copy.
        TEST: Tests that a value at the target location is equal to a
            specified value. The operation object must contain a value
            parameter that defines the value to compare to the target
            location's value. For the operation to succeed, the target
            location must be equal to the value value. For test, equal
            indicates that the value at the target location and the value that
            value defines are of the same JSON type. The data type of the
            value determines how equality is defined: Type Considered equal if
            both values strings Contain the same number of Unicode characters
            and their code points are byte-by-byte equal. numbers Are
            numerically equal. arrays Contain the same number of values, and
            each value is equal to the value at the corresponding position in
            the other array, by using these type-specific rules. objects
            Contain the same number of parameters, and each parameter is equal
            to a parameter in the other object, by comparing their keys (as
            strings) and their values (by using these type-specific rules).
            literals (false, true, and null) Are the same. The comparison is a
            logical comparison. For example, whitespace between the parameter
            values of an array is not significant. Also, ordering of the
            serialization of object parameters is not significant. For more
            information, see 4.6. test.

    """
    ADD = 'add'

    REMOVE = 'remove'

    REPLACE = 'replace'

    MOVE = 'move'

    COPY = 'copy'

    TEST = 'test'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class Frequency(object):

    """Implementation of the 'Frequency' model.

    The frequency of the billing cycle.

    Attributes:
        interval_unit (IntervalUnit): The interval at which the subscription
            is charged or billed.
        interval_count (int): The number of intervals after which a subscriber
            is billed. For example, if the `interval_unit` is `DAY` with an
            `interval_count` of  `2`, the subscription is billed once every
            two days. The following table lists the maximum allowed values for
            the `interval_count` for each `interval_unit`: Interval unit
            Maximum interval count DAY 365 WEEK 52 MONTH 12 YEAR 1

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "interval_unit": 'interval_unit',
        "interval_count": 'interval_count'
    }

    _optionals = [
        'interval_count',
    ]

    def __init__(self,
                 interval_unit=None,
                 interval_count=1):
        """Constructor for the Frequency class"""

        # Initialize members of the class
        self.interval_unit = interval_unit 
        self.interval_count = interval_count 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        interval_unit = dictionary.get("interval_unit") if dictionary.get("interval_unit") else None
        interval_count = dictionary.get("interval_count") if dictionary.get("interval_count") else 1
        # Return an object of this model
        return cls(interval_unit,
                   interval_count)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'interval_unit={self.interval_unit!r}, '
                f'interval_count={(self.interval_count if hasattr(self, "interval_count") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'interval_unit={self.interval_unit!s}, '
                f'interval_count={(self.interval_count if hasattr(self, "interval_count") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.link_description import LinkDescription
from paypalserversdk.models.subscription import Subscription


class SubscriptionCollection(object):

    """Implementation of the 'Subscription Collection' model.

    The list of subscriptions.

    Attributes:
        subscriptions (List[Subscription]): An array of subscriptions.
        links (List[LinkDescription]): An array of request-related [HATEOAS
            links](/docs/api/reference/api-responses/#hateoas-links).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "subscriptions": 'subscriptions',
        "links": 'links'
    }

    _optionals = [
        'subscriptions',
        'links',
    ]

    def __init__(self,
                 subscriptions=APIHelper.SKIP,
                 links=APIHelper.SKIP):
        """Constructor for the SubscriptionCollection class"""

        # Initialize members of the class
        if subscriptions is not APIHelper.SKIP:
            self.subscriptions = subscriptions 
        if links is not APIHelper.SKIP:
            self.links = links 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        subscriptions = None
        if dictionary.get('subscriptions') is not None:
            subscriptions = [Subscription.from_dictionary(x) for x in dictionary.get('subscriptions')]
        else:
            subscriptions = APIHelper.SKIP
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        # Return an object of this model
        return cls(subscriptions,
                   links)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'subscriptions={(self.subscriptions if hasattr(self, "subscriptions") else None)!r}, '
                f'links={(self.links if hasattr(self, "links") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'subscriptions={(self.subscriptions if hasattr(self, "subscriptions") else None)!s}, '
                f'links={(self.links if hasattr(self, "links") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class RefundStatus(object):

    """Implementation of the 'Refund Status' enum.

    The status of the refund.

    Attributes:
        CANCELLED: The refund was cancelled.
        FAILED: The refund could not be processed.
        PENDING: The refund is pending. For more information, see
            status_details.reason.
        COMPLETED: The funds for this transaction were debited to the
            customer's account.

    """
    CANCELLED = 'CANCELLED'

    FAILED = 'FAILED'

    PENDING = 'PENDING'

    COMPLETED = 'COMPLETED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CaptureIncompleteReason(object):

    """Implementation of the 'Capture Incomplete Reason' enum.

    The reason why the captured payment status is `PENDING` or `DENIED`.

    Attributes:
        BUYER_COMPLAINT: The payer initiated a dispute for this captured
            payment with PayPal.
        CHARGEBACK: The captured funds were reversed in response to the payer
            disputing this captured payment with the issuer of the financial
            instrument used to pay for this captured payment.
        ECHECK: The payer paid by an eCheck that has not yet cleared.
        INTERNATIONAL_WITHDRAWAL: Visit your online account. In your **Account
            Overview**, accept and deny this payment.
        OTHER: No additional specific reason can be provided. For more
            information about this captured payment, visit your account online
            or contact PayPal.
        PENDING_REVIEW: The captured payment is pending manual review.
        RECEIVING_PREFERENCE_MANDATES_MANUAL_ACTION: The payee has not yet set
            up appropriate receiving preferences for their account. For more
            information about how to accept or deny this payment, visit your
            account online. This reason is typically offered in scenarios such
            as when the currency of the captured payment is different from the
            primary holding currency of the payee.
        REFUNDED: The captured funds were refunded.
        TRANSACTION_APPROVED_AWAITING_FUNDING: The payer must send the funds
            for this captured payment. This code generally appears for manual
            EFTs.
        UNILATERAL: The payee does not have a PayPal account.
        VERIFICATION_REQUIRED: The payee's PayPal account is not verified.
        DECLINED_BY_RISK_FRAUD_FILTERS: Risk Filter set by the payee failed
            for the transaction.

    """
    BUYER_COMPLAINT = 'BUYER_COMPLAINT'

    CHARGEBACK = 'CHARGEBACK'

    ECHECK = 'ECHECK'

    INTERNATIONAL_WITHDRAWAL = 'INTERNATIONAL_WITHDRAWAL'

    OTHER = 'OTHER'

    PENDING_REVIEW = 'PENDING_REVIEW'

    RECEIVING_PREFERENCE_MANDATES_MANUAL_ACTION = 'RECEIVING_PREFERENCE_MANDATES_MANUAL_ACTION'

    REFUNDED = 'REFUNDED'

    TRANSACTION_APPROVED_AWAITING_FUNDING = 'TRANSACTION_APPROVED_AWAITING_FUNDING'

    UNILATERAL = 'UNILATERAL'

    VERIFICATION_REQUIRED = 'VERIFICATION_REQUIRED'

    DECLINED_BY_RISK_FRAUD_FILTERS = 'DECLINED_BY_RISK_FRAUD_FILTERS'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.address import Address
from paypalserversdk.models.phone_number_with_country_code import PhoneNumberWithCountryCode
from paypalserversdk.models.shipping_name import ShippingName


class VaultedDigitalWalletShippingDetails(object):

    """Implementation of the 'Vaulted Digital Wallet Shipping Details' model.

    The shipping details.

    Attributes:
        name (ShippingName): The name of the party.
        email_address (str): The internationalized email address. Note: Up to
            64 characters are allowed before and 255 characters are allowed
            after the @ sign. However, the generally accepted maximum length
            for an email address is 254 characters. The pattern verifies that
            an unquoted @ sign exists.
        phone_number (PhoneNumberWithCountryCode): The phone number, in its
            canonical international [E.164 numbering plan
            format](https://www.itu.int/rec/T-REC-E.164/en).
        mtype (FulfillmentType): A classification for the method of purchase
            fulfillment (e.g shipping, in-store pickup, etc). Either `type` or
            `options` may be present, but not both.
        address (Address): The portable international postal address. Maps to
            [AddressValidationMetadata](https://github.com/googlei18n/libaddres
            sinput/wiki/AddressValidationMetadata) and HTML 5.1 [Autofilling
            form controls: the autocomplete
            attribute](https://www.w3.org/TR/html51/sec-forms.html#autofilling-
            form-controls-the-autocomplete-attribute).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "email_address": 'email_address',
        "phone_number": 'phone_number',
        "mtype": 'type',
        "address": 'address'
    }

    _optionals = [
        'name',
        'email_address',
        'phone_number',
        'mtype',
        'address',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 email_address=APIHelper.SKIP,
                 phone_number=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 address=APIHelper.SKIP):
        """Constructor for the VaultedDigitalWalletShippingDetails class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if email_address is not APIHelper.SKIP:
            self.email_address = email_address 
        if phone_number is not APIHelper.SKIP:
            self.phone_number = phone_number 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if address is not APIHelper.SKIP:
            self.address = address 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        name = ShippingName.from_dictionary(dictionary.get('name')) if 'name' in dictionary.keys() else APIHelper.SKIP
        email_address = dictionary.get("email_address") if dictionary.get("email_address") else APIHelper.SKIP
        phone_number = PhoneNumberWithCountryCode.from_dictionary(dictionary.get('phone_number')) if 'phone_number' in dictionary.keys() else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        address = Address.from_dictionary(dictionary.get('address')) if 'address' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   email_address,
                   phone_number,
                   mtype,
                   address)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'email_address={(self.email_address if hasattr(self, "email_address") else None)!r}, '
                f'phone_number={(self.phone_number if hasattr(self, "phone_number") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r}, '
                f'address={(self.address if hasattr(self, "address") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'email_address={(self.email_address if hasattr(self, "email_address") else None)!s}, '
                f'phone_number={(self.phone_number if hasattr(self, "phone_number") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s}, '
                f'address={(self.address if hasattr(self, "address") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from paypalserversdk.api_helper import APIHelper
from paypalserversdk.configuration import Server
from paypalserversdk.http.api_response import ApiResponse
from paypalserversdk.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from paypalserversdk.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from paypalserversdk.models.payment_authorization import PaymentAuthorization
from paypalserversdk.models.captured_payment import CapturedPayment
from paypalserversdk.models.refund import Refund
from paypalserversdk.exceptions.error_exception import ErrorException
from paypalserversdk.exceptions.api_exception import ApiException


class PaymentsController(BaseController):

    """A Controller to access Endpoints in the paypalserversdk API."""
    def __init__(self, config):
        super(PaymentsController, self).__init__(config)

    def get_authorized_payment(self,
                               options=dict()):
        """Does a GET request to /v2/payments/authorizations/{authorization_id}.

        Shows details for an authorized payment, by ID.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    authorization_id -- str -- The ID of the authorized
                        payment for which to show details.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see
                        [PayPal-Auth-Assertion](/docs/api/reference/api-request
                        s/#paypal-auth-assertion). Note:For three party
                        transactions in which a partner is managing the API
                        calls on behalf of a merchant, the partner must
                        identify the merchant using either a
                        PayPal-Auth-Assertion header or an access token with
                        target_subject.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP 200 OK status code and a
                JSON response body that shows authorization details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/authorizations/{authorization_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('authorization_id')
                            .value(options.get('authorization_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaymentAuthorization.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', ApiException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def capture_authorized_payment(self,
                                   options=dict()):
        """Does a POST request to /v2/payments/authorizations/{authorization_id}/capture.

        Captures an authorized payment, by ID.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    authorization_id -- str -- The PayPal-generated ID for the
                        authorized payment to capture.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_request_id -- str -- The server stores keys for 45
                        days.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see
                        [PayPal-Auth-Assertion](/docs/api/reference/api-request
                        s/#paypal-auth-assertion). Note:For three party
                        transactions in which a partner is managing the API
                        calls on behalf of a merchant, the partner must
                        identify the merchant using either a
                        PayPal-Auth-Assertion header or an access token with
                        target_subject.
                    body -- CaptureRequest -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP 200 OK status code and a
                JSON response body that shows captured payment details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/authorizations/{authorization_id}/capture')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('authorization_id')
                            .value(options.get('authorization_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CapturedPayment.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'The request failed because it is not well-formed or is syntactically incorrect or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('409', 'The server has detected a conflict while processing this request.', ErrorException)
            .local_error('422', 'The request failed because it is semantically incorrect or failed business validation.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', ApiException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def reauthorize_payment(self,
                            options=dict()):
        """Does a POST request to /v2/payments/authorizations/{authorization_id}/reauthorize.

        Reauthorizes an authorized PayPal account payment, by ID. To ensure
        that funds are still available, reauthorize a payment after its
        initial three-day honor period expires. Within the 29-day
        authorization period, you can issue multiple re-authorizations after
        the honor period expires. If 30 days have transpired since the date of
        the original authorization, you must create an authorized payment
        instead of reauthorizing the original authorized payment. A
        reauthorized payment itself has a new honor period of three days. You
        can reauthorize an authorized payment from 4 to 29 days after the
        3-day honor period. The allowed amount depends on context and
        geography, for example in US it is up to 115% of the original
        authorized amount, not to exceed an increase of $75 USD. Supports only
        the `amount` request parameter.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    authorization_id -- str -- The PayPal-generated ID for the
                        authorized payment to reauthorize.
                    paypal_request_id -- str -- The server stores keys for 45
                        days.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see
                        [PayPal-Auth-Assertion](/docs/api/reference/api-request
                        s/#paypal-auth-assertion). Note:For three party
                        transactions in which a partner is managing the API
                        calls on behalf of a merchant, the partner must
                        identify the merchant using either a
                        PayPal-Auth-Assertion header or an access token with
                        target_subject.
                    body -- ReauthorizeRequest -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP 200 OK status code and a
                JSON response body that shows the reauthorized payment details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/authorizations/{authorization_id}/reauthorize')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('authorization_id')
                            .value(options.get('authorization_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaymentAuthorization.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'The request failed because it is not well-formed or is syntactically incorrect or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('422', 'The request failed because it either is semantically incorrect or failed business validation.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', ApiException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def void_payment(self,
                     options=dict()):
        """Does a POST request to /v2/payments/authorizations/{authorization_id}/void.

        Voids, or cancels, an authorized payment, by ID. You cannot void an
        authorized payment that has been fully captured.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    authorization_id -- str -- The PayPal-generated ID for the
                        authorized payment to void.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see
                        [PayPal-Auth-Assertion](/docs/api/reference/api-request
                        s/#paypal-auth-assertion). Note:For three party
                        transactions in which a partner is managing the API
                        calls on behalf of a merchant, the partner must
                        identify the merchant using either a
                        PayPal-Auth-Assertion header or an access token with
                        target_subject.
                    paypal_request_id -- str -- The server stores keys for 45
                        days.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP 200 OK status code and a
                JSON response body that shows authorization details. This
                response is returned when the Prefer header is set to
                return=representation.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/authorizations/{authorization_id}/void')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('authorization_id')
                            .value(options.get('authorization_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaymentAuthorization.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('409', 'The request failed because a previous call for the given resource is in progress.', ErrorException)
            .local_error('422', 'The request failed because it either is semantically incorrect or failed business validation.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', ApiException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def get_captured_payment(self,
                             options=dict()):
        """Does a GET request to /v2/payments/captures/{capture_id}.

        Shows details for a captured payment, by ID.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    capture_id -- str -- The PayPal-generated ID for the
                        captured payment for which to show details.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP 200 OK status code and a
                JSON response body that shows captured payment details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/captures/{capture_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('capture_id')
                            .value(options.get('capture_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CapturedPayment.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', ApiException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def refund_captured_payment(self,
                                options=dict()):
        """Does a POST request to /v2/payments/captures/{capture_id}/refund.

        Refunds a captured payment, by ID. For a full refund, include an empty
        payload in the JSON request body. For a partial refund, include an
        amount object in the JSON request body.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    capture_id -- str -- The PayPal-generated ID for the
                        captured payment to refund.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_request_id -- str -- The server stores keys for 45
                        days.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value is:
                        return=minimal. The server returns a minimal response
                        to optimize communication between the API caller and
                        the server. A minimal response includes the id, status
                        and HATEOAS links. return=representation. The server
                        returns a complete resource representation, including
                        the current state of the resource.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see
                        [PayPal-Auth-Assertion](/docs/api/reference/api-request
                        s/#paypal-auth-assertion). Note:For three party
                        transactions in which a partner is managing the API
                        calls on behalf of a merchant, the partner must
                        identify the merchant using either a
                        PayPal-Auth-Assertion header or an access token with
                        target_subject.
                    body -- RefundRequest -- The request body parameter.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP 200 OK status code and a
                JSON response body that shows refund details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/captures/{capture_id}/refund')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('capture_id')
                            .value(options.get('capture_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('paypal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Refund.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'The request failed because it is not well-formed or is syntactically incorrect or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('409', 'The request failed because a previous call for the given resource is in progress.', ErrorException)
            .local_error('422', 'The request failed because it either is semantically incorrect or failed business validation.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', ApiException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def get_refund(self,
                   options=dict()):
        """Does a GET request to /v2/payments/refunds/{refund_id}.

        Shows details for a refund, by ID.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    refund_id -- str -- The PayPal-generated ID for the refund
                        for which to show details.
                    paypal_mock_response -- str -- PayPal's REST API uses a
                        request header to invoke negative testing in the
                        sandbox. This header configures the sandbox into a
                        negative testing state for transactions that include
                        the merchant.
                    paypal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see
                        [PayPal-Auth-Assertion](/docs/api/reference/api-request
                        s/#paypal-auth-assertion). Note:For three party
                        transactions in which a partner is managing the API
                        calls on behalf of a merchant, the partner must
                        identify the merchant using either a
                        PayPal-Auth-Assertion header or an access token with
                        target_subject.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP 200 OK status code and a
                JSON response body that shows refund details.

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/refunds/{refund_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('refund_id')
                            .value(options.get('refund_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('PayPal-Mock-Response')
                          .value(options.get('paypal_mock_response', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('paypal_auth_assertion', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Refund.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', ApiException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()
